module TETRIS (CLK, VH, VRGB, KEYS, RIGHT, LEFT, DOWN, ROTATE, ROUT, GOUT, BOUT) ;

input CLK ;
input VH ;
input VRGB;  // ysynch
input [6:0] KEYS;  //keyboard input
input RIGHT;
input LEFT;
input DOWN;
input ROTATE;
output ROUT ;
output GOUT ;
output BOUT ;

// Tetris declarations
reg [135:0] ROW;

// active piece
reg [3:0] activex;  // 3-bits -- horizontal, 4-bits -- vertical
reg [6:0] activey;  // 3-bits -- horizontal, 4-bits -- vertical

reg [3:0] activex2;  // treat directional offsets as actual blocks
reg [6:0] activey2;
reg [3:0] activex3;
reg [6:0] activey3;
reg [3:0] activex4;
reg [6:0] activey4;


reg [1:0] active2;  // 2-bits for additional piece
reg [1:0] active3;  // 2-bits for additional piece
reg [1:0] active4;  // 2-bits for additional piece

reg [2:0] state;   //determine if we need a new active piece

reg [4:0] piece;     // used to simulate random piece generation

reg [9:0] Xcounter;
reg [9:0] Ycounter;

reg [3:0] tempX;	//temp holding vars to split up shifting and compares
reg [6:0] tempY;

//reg [6:0] box;

reg [7:0] FromLower;
reg [7:0] ToLower;


reg [2:0] RowCount;
reg [3:0] Counter;
reg [5:0] ActiveCounter;
reg [7:0] ActiveSpot;
reg [7:0] ActiveNext;
reg [7:0] ActiveSpot2;
reg [7:0] ActiveNext2;
reg [7:0] ActiveSpot3;
reg [7:0] ActiveNext3;
reg [7:0] ActiveSpot4;
reg [7:0] ActiveNext4;

reg [1:0] RightFlag;
reg [1:0] LeftFlag;
reg [1:0] DownFlag;
reg [1:0] RotateFlag;

//reg R;
//reg G;
//reg B;

reg [2:0] RGB;
reg myKey;

assign ROUT = RGB[0];
assign GOUT = RGB[1];
assign BOUT = RGB[2];


// Welcome to Tetris
always@(posedge CLK) begin

if(!VRGB) begin
  Xcounter <= 0; 
  Ycounter <= 0;

//  ROW[123] <= 1;
//  ROW[135:128] <= 511;
  ROW[128] <= 1;
  ROW[129] <= 1;
  ROW[130] <= 1;
  ROW[131] <= 1;
  ROW[132] <= 1;
  ROW[133] <= 1;
  ROW[134] <= 1;
  ROW[135] <= 1;
  
end
else
if(VH) begin

  if(Xcounter == 1 && Ycounter == 1) begin
    Counter <= Counter + 1;
    ActiveCounter <= 0;  
    RowCount <= 0;
  end

  if(Xcounter > 253 && Xcounter < 383 && Ycounter > 111 && Ycounter < 368)
  begin
  //draw board to screen

  tempX <= (Xcounter - 254) >> 4;
  tempY <= (Ycounter - 112) >> 4;

	if(Xcounter == 254) begin
		RGB = 2;
//		R <= 0;  // draw green background (hack to fix red piece overflow)
// 	  	G <= 1;
//	        B <= 0;
	end
	else begin

		if(ROW[(tempY << 3) | tempX])
			begin
  			   RGB = 1;
			   //R <= 1;  // draw red dead pieces
	 	  	   //G <= 0;
			   //B <= 0;
			end
		else begin
		
			if(((tempY == activey) && (tempX == activex)) || ((tempY == activey2) && (tempX == activex2)) || ((tempY == activey3) && (tempX == activex3)) || ((tempY == activey4) && (tempX == activex4))) begin
  		           RGB = 3;
//			   R <= 1;  // draw yellow active piece
//	 	  	   G <= 1;
//			   B <= 0;
			end
		
			else
			begin
			//if(KEYS > 42)
			  // RGB = 4;
			   //else
                           RGB = 0;
//			   R <= 0;  // draw black empty square
//			   G <= 0;
//			   B <= 0;
			end
		end
	end
end
else //draw green and calculate next game state;

begin
	RGB = 2;

//	R <= 0;  // draw green background
//	G <= 1;	
//	B <= 0;
  
case(state)

0:begin

	case(RowCount)
	0: begin

		activex <= 3;
		activey <= 0;

		ToLower <= 120;
		FromLower <= 120;

		case(piece)
		0:begin  //4 long
			active2 <= 2;
			active3 <= 2;
			active4 <= 2;
		end
		
		1:begin //box
			active2 <= 3;
			active3 <= 2;
			active4 <= 1;
		end
		2:begin //left L
			active2 <= 2;
			active3 <= 2;
			active4 <= 1;
		end
		3:begin //right L
			active2 <= 2;
			active3 <= 2;
			active4 <= 3;
		end
		4:begin //right Z
			active2 <= 2;
			active3 <= 3;
			active4 <= 2;
		end
		5:begin //left Z
			active2 <= 2;
			active3 <= 1;
			active4 <= 2;
		end
		6:begin //bonus suprise piece
			active2 <= 1;
			active3 <= 2;
			active4 <= 3;		
		end

		endcase
		
		RowCount <= 1;
	end
	
	1: begin
	
		if(ToLower == 0) begin  // we be done checkin' rows
			RowCount <= 0;  // get ready for next piece
			state <= 1;
		end
		
		else begin
			if((ROW[ToLower] == 0) || (ROW[ToLower+1] == 0) || (ROW[ToLower+2] == 0) || (ROW[ToLower+3] == 0) || (ROW[ToLower+4] == 0) || (ROW[ToLower+5] == 0) || (ROW[ToLower+6] == 0) || (ROW[ToLower+7] == 0)) begin
  			  //if row is not full
				ToLower <= ToLower - 8;
			end
			  
			if(FromLower != 0) begin

				FromLower <= FromLower - 8;

			end
			RowCount <= 2;
		end
	end
	2: begin
		if(FromLower != 0) begin

			ROW[ToLower] <= ROW[FromLower];
			ROW[ToLower+1] <= ROW[FromLower+1];
			ROW[ToLower+2] <= ROW[FromLower+2];
			ROW[ToLower+3] <= ROW[FromLower+3];				
			ROW[ToLower+4] <= ROW[FromLower+4];
			ROW[ToLower+5] <= ROW[FromLower+5];				
			ROW[ToLower+6] <= ROW[FromLower+6];
			ROW[ToLower+7] <= ROW[FromLower+7];				
			
		end
		else begin
			ROW[ToLower] <= 0;
			ROW[ToLower+1] <= 0;
			ROW[ToLower+2] <= 0;
			ROW[ToLower+3] <= 0;								
			ROW[ToLower+4] <= 0;
			ROW[ToLower+5] <= 0;								
			ROW[ToLower+6] <= 0;
			ROW[ToLower+7] <= 0;								
		end
		
		RowCount <= 1;
	end
			
	endcase
end

1:begin

if(RIGHT && (RightFlag == 0)) begin
	RightFlag <= 1;
end
else if(!RIGHT)
begin
	RightFlag <= 0;	
end


if(LEFT && (LeftFlag == 0)) begin
	LeftFlag <= 1;
end
else if(!LEFT)
begin
	LeftFlag <= 0;	
end


if(DOWN && (DownFlag == 0)) begin
	DownFlag <= 1;
end
else if(!DOWN)
begin
	DownFlag <= 0;	
end


if(ROTATE && (RotateFlag == 0)) begin
	RotateFlag <= 1;
end
else if(!ROTATE)
begin
	RotateFlag <= 0;	
end


	case(ActiveCounter)
	
	0: begin
		if(active2 == 0 || active2 == 2) begin
			activex2 <= activex;
			activey2 <= activey + active2 - 1;
		end
		else begin
			activex2 <= activex + active2 - 2;
			activey2 <= activey;
 	     	end
	     	ActiveCounter <= 1;
	   end
		
	1:begin
		if(active3 == 0 || active3 == 2) begin
			activex3 <= activex2;
			activey3 <= activey2 + active3 - 1;
		end
		else begin
			activex3 <= activex2 + active3 - 2;
			activey3 <= activey2;
		end
     		ActiveCounter <= 2;
	end	
	2:begin
		if(active4 == 0 || active4 == 2) begin
			activex4 <= activex3;
			activey4 <= activey3 + active4 - 1;

		end
		else begin
			activex4 <= activex3 + active4- 2;
			activey4 <= activey3;

		end
     		ActiveCounter <= 3;
	end
	
	3:begin
		ActiveSpot <= ((activey << 3) | activex);
		ActiveCounter <= 4;
	  end

	4: begin
   		
	if(active2 == 0 || active2 == 2) begin
		ActiveSpot2 = ActiveSpot + ((active2-1)*8);
		end
	else begin
		ActiveSpot2 = ActiveSpot + (active2-2);
 	     end
 	     
		ActiveCounter <= 5;
	end

	5: begin
	if(active3 == 0 || active3 == 2) begin
		ActiveSpot3 = ActiveSpot2 + ((active3-1)*8);
		end
	else begin
		ActiveSpot3 = ActiveSpot2 + (active3-2);
 	     end
		ActiveCounter <= 6;
	end	
	
	6: begin
	if(active4 == 0 || active4 == 2) begin
		ActiveSpot4 = ActiveSpot3+ ((active4-1)*8);
		end
	else begin
		ActiveSpot4 = ActiveSpot3 + (active4-2);
 	     end
	
	ActiveCounter <= 7;
	
	end	
	
	//check left/right/down, if OK then move

	7:begin
  		if(RightFlag == 1) begin
  			if((activex != 7) && (activex2 != 7) && (activex3 != 7) && (activex4 != 7) && (ROW[ActiveSpot + 1] != 1) && (ROW[ActiveSpot2 + 1] != 1) && (ROW[ActiveSpot3 + 1] != 1) && (ROW[ActiveSpot4 + 1] != 1))
  			activex <= activex + 1;
  			//check for boundary conditions
  			RightFlag <= 2;
  		end
  		
  	  	if(LeftFlag == 1) begin
  	  		if((activex != 0) && (activex2 != 0) && (activex3 != 0) && (activex4 != 0) && (ROW[ActiveSpot - 1] != 1) && (ROW[ActiveSpot2 - 1] != 1) && (ROW[ActiveSpot3 - 1] != 1) && (ROW[ActiveSpot4 - 1] != 1))
  			activex <= activex - 1;
  			//check for boundary conditions
  			LeftFlag <= 2;
  		end	
  	  	if(DownFlag == 1) begin
  			if((ROW[ActiveSpot + 8] != 1) && (ROW[ActiveSpot2 + 8] != 1) && (ROW[ActiveSpot3 + 8] != 1) && (ROW[ActiveSpot4 + 8] != 1))
  			activey <= activey + 1;
  			//check for boundary conditions
  			DownFlag <= 2;
  		end

	ActiveCounter <= 8;
	end

	8: begin
		active2 <= active2 + 1;
		active3 <= active3 + 1;
		active4 <= active4 + 1;			
		ActiveCounter <= 9;
	end
	
	9: begin
		if(active2 == 0 || active2 == 2) begin
			activex2 <= activex;
			activey2 <= activey + active2 - 1;
		end
		else begin
			activex2 <= activex + active2 - 2;
			activey2 <= activey;
 	     	end
	     	ActiveCounter <= 10;
	   end
		
	10:begin
		if(active3 == 0 || active3 == 2) begin
			activex3 <= activex2;
			activey3 <= activey2 + active3 - 1;
		end
		else begin
			activex3 <= activex2 + active3 - 2;
			activey3 <= activey2;
		end
     		ActiveCounter <= 11;
	end	
	11:begin
		if(active4 == 0 || active4 == 2) begin
			activex4 <= activex3;
			activey4 <= activey3 + active4 - 1;

		end
		else begin
			activex4 <= activex3 + active4- 2;
			activey4 <= activey3;

		end
     		ActiveCounter <= 12;
	end
	
	12:begin
		ActiveSpot <= ((activey << 3) | activex);
		ActiveCounter <= 13;
	  end

	13: begin
   		
	if(active2 == 0 || active2 == 2) begin
		ActiveSpot2 = ActiveSpot + ((active2-1)*8);
		end
	else begin
		ActiveSpot2 = ActiveSpot + (active2-2);
 	     end
 	     
		ActiveCounter <= 14;
	end

	14: begin
	if(active3 == 0 || active3 == 2) begin
		ActiveSpot3 = ActiveSpot2 + ((active3-1)*8);
		end
	else begin
		ActiveSpot3 = ActiveSpot2 + (active3-2);
 	     end
		ActiveCounter <= 15;
	end	
	
	15: begin
	if(active4 == 0 || active4 == 2) begin
		ActiveSpot4 = ActiveSpot3+ ((active4-1)*8);
		end
	else begin
		ActiveSpot4 = ActiveSpot3 + (active4-2);
 	     end
	
	ActiveCounter <= 16;
	
	end	
	
	16:begin
		active2 <= active2 - 1;
		active3 <= active3 - 1;
		active4 <= active4 - 1;			
		ActiveCounter <= 17;
	  end

	//check rotation here and then rotate if OK


	17: begin 		// KEY recognition phase
  		
  	  	if(RotateFlag == 1) begin
  	  		if((ROW[ActiveSpot] != 1) && (ROW[ActiveSpot2] != 1) && (ROW[ActiveSpot3] != 1) && (ROW[ActiveSpot4] != 1) && (activex >= 0) && (activex2 >= 0) &&(activex3 >= 0) &&(activex4 >= 0) && (activex < 8) && (activex2 < 8) &&(activex3 < 8) &&(activex4 < 8)) begin
			active2 <= active2 + 1;
			active3 <= active3 + 1;
			active4 <= active4 + 1;			
  			end
		
			RotateFlag <= 2;
  		end
  				  	
	  	ActiveCounter <= 18;
 	 end
	18: begin
		if(active2 == 0 || active2 == 2) begin
			activex2 <= activex;
			activey2 <= activey + active2 - 1;
		end
		else begin
			activex2 <= activex + active2 - 2;
			activey2 <= activey;
 	     	end
	     	ActiveCounter <= 19;
	   end
		
	19:begin
		if(active3 == 0 || active3 == 2) begin
			activex3 <= activex2;
			activey3 <= activey2 + active3 - 1;
		end
		else begin
			activex3 <= activex2 + active3 - 2;
			activey3 <= activey2;
		end
     		ActiveCounter <= 20;
	end	
	20:begin
		if(active4 == 0 || active4 == 2) begin
			activex4 <= activex3;
			activey4 <= activey3 + active4 - 1;

		end
		else begin
			activex4 <= activex3 + active4- 2;
			activey4 <= activey3;

		end
     		ActiveCounter <= 21;
	end
	
	21:begin
		ActiveSpot <= ((activey << 3) | activex);
		ActiveCounter <= 22;
	  end

	22: begin
   		
	if(active2 == 0 || active2 == 2) begin
		ActiveSpot2 = ActiveSpot + ((active2-1)*8);
		end
	else begin
		ActiveSpot2 = ActiveSpot + (active2-2);
 	     end
 	     
		ActiveCounter <= 23;
	end
	23: begin
	if(active3 == 0 || active3 == 2) begin
		ActiveSpot3 = ActiveSpot2 + ((active3-1)*8);
		end
	else begin
		ActiveSpot3 = ActiveSpot2 + (active3-2);
 	     end
		ActiveCounter <= 24;
	end	
	24: begin
	if(active4 == 0 || active4 == 2) begin
		ActiveSpot4 = ActiveSpot3+ ((active4-1)*8);
		end
	else begin
		ActiveSpot4 = ActiveSpot3 + (active4-2);
 	     end
		ActiveCounter <= 25;
	end	
	
	25:begin
		ActiveNext <= ActiveSpot + 8;
		ActiveNext2 <= ActiveSpot2 + 8;
		ActiveNext3 <= ActiveSpot3 + 8;
		ActiveNext4 <= ActiveSpot4 + 8;	   
		ActiveCounter <= 26;
	  end

	26:begin
	
	if(Counter == 15) begin
	     if((ROW[ActiveNext]) || (ROW[ActiveNext2]) || (ROW[ActiveNext3]) || (ROW[ActiveNext4])) begin
	     	ROW[ActiveSpot] <= 1;
		ROW[ActiveSpot2] <= 1;
		ROW[ActiveSpot3] <= 1;
		ROW[ActiveSpot4] <= 1;		
		
		if(activey == 0) begin
			ROW <= 0;  // clear board
		end
		
		state <= 0;
	
	     end
	     else begin
		   activey <= activey + 1;
	     end
             Counter <= 0;
   	end
   	  
	ActiveCounter <= 27;
	
	end
	endcase
    end
  endcase
end  //end calculate game state

//see if we are off the drawing area, if we are, reset.
if(Xcounter < 639) 
  begin
  Xcounter <= Xcounter + 1;
  end
else
  begin
    Xcounter <= 0;
    Ycounter <= Ycounter + 1;
  end
end
 
//"randomize" next piece to choose kind of
if(piece < 6)
  piece <= piece + 1;
else
  piece <= 0;  

end
endmodule 
